@@ -1,5 +1,4 @@
 import sys
-import importlib
 
 from .utils import State
 from .applications import Index
@@ -7,11 +6,8 @@ from .config import here, Config
 
 __all__ = ["Index", "Config", "g"]
 
-# default app
-app = Index()
+# Current working directory first
+sys.path.insert(0, here)
 
 # global state
 g = State()
-
-# Current working directory first
-sys.path.insert(0, here)
