@@ -2,58 +2,78 @@ import os
 import copy
 import typing
 import asyncio
+import logging
 import traceback
 import importlib
 from types import ModuleType
 
-from starlette.types import Scope, Receive, Send, ASGIApp, Message
 from starlette.status import WS_1001_GOING_AWAY
-from starlette.requests import URL
+from starlette.datastructures import URL
 from starlette.websockets import WebSocketClose
-from starlette.staticfiles import StaticFiles
-from starlette.responses import RedirectResponse
 from starlette.middleware import Middleware
 from starlette.middleware.cors import CORSMiddleware
 from starlette.middleware.trustedhost import TrustedHostMiddleware
-from starlette.routing import NoMatchFound
-from jinja2 import Environment, FileSystemLoader
+from jinja2 import Environment, ChoiceLoader, FileSystemLoader, PackageLoader
 from a2wsgi import WSGIMiddleware
 
-from .types import WSGIApp
-from .utils import Singleton
-from .config import here, Config
+from .types import WSGIApp, Scope, Receive, Send, ASGIApp, Message, Literal, TypedDict
+from .utils import cached_property
+from .config import Config
+from .routing import Router, BaseRoute, NoMatchFound
+from .http import responses
+from .http.view import parse_params
 from .http.debug import ServerErrorMiddleware
 from .http.request import Request
-from .http.responses import Response, FileResponse, TemplateResponse, PlainTextResponse
-from .http.background import (
-    BackgroundTasks,
-    after_response_tasks_var,
-    finished_response_tasks_var,
+from .http.responses import (
+    convert,
+    Response,
+    FileResponse,
+    RedirectResponse,
 )
 from .http.exceptions import HTTPException, ExceptionMiddleware
 from .websocket.request import WebSocket
-from .preset import (
-    check_on_startup,
-    clear_check_on_shutdown,
-    create_directories,
-    clear_directories,
-)
+
+
+logger = logging.getLogger(__name__)
+
+
+class FactoryClass(TypedDict):
+    """
+    `Index().factory_class` type
+    """
+
+    http: typing.Type[Request]
+    websocket: typing.Type[WebSocket]
+
+
+def try_html(request: Request) -> typing.Optional[Response]:
+    """
+    try find html through TemplateResponse
+    """
+    try:
+        return responses.TemplateResponse(
+            request["path"] + ".html", {"request": request}
+        )
+    except LookupError:
+        return None
 
 
 class Lifespan:
     def __init__(
         self,
-        on_startup: typing.Dict[str, typing.Callable] = None,
-        on_shutdown: typing.Dict[str, typing.Callable] = None,
+        on_startup: typing.List[typing.Callable] = None,
+        on_shutdown: typing.List[typing.Callable] = None,
     ) -> None:
-        self.on_startup: typing.Dict[str, typing.Callable] = on_startup or {}
-        self.on_shutdown: typing.Dict[str, typing.Callable] = on_shutdown or {}
+        self.on_startup = on_startup or []
+        self.on_shutdown = on_shutdown or []
 
-    def add_event_handler(self, event_type: str, func: typing.Callable) -> None:
+    def add_event_handler(
+        self, event_type: Literal["startup", "shutdown"], func: typing.Callable
+    ) -> None:
         if event_type == "startup":
-            self.on_startup[func.__qualname__] = func
+            self.on_startup.append(func)
         elif event_type == "shutdown":
-            self.on_shutdown[func.__qualname__] = func
+            self.on_shutdown.append(func)
         else:
             raise ValueError("event_type must be in ('startup', 'shutdown')")
 
@@ -61,7 +81,7 @@ class Lifespan:
         """
         Run any `.on_startup` event handlers.
         """
-        for handler in self.on_startup.values():
+        for handler in self.on_startup:
             if asyncio.iscoroutinefunction(handler):
                 await handler()
             else:
@@ -71,7 +91,7 @@ class Lifespan:
         """
         Run any `.on_shutdown` event handlers.
         """
-        for handler in self.on_shutdown.values():
+        for handler in self.on_shutdown:
             if asyncio.iscoroutinefunction(handler):
                 await handler()
             else:
@@ -104,18 +124,26 @@ class Lifespan:
 
 
 class IndexFile:
+    """
+    TODO: need change
+    """
+
     def __init__(
-        self,
-        module_name: str,
-        basepath: str,
-        *,
-        try_html: bool = False,
-        factory_class=None,
+        self, module_name: str, basepath: str = None, allow_underline: bool = False
     ) -> None:
         self.module_name = module_name
-        self.basepath = basepath
-        self.try_html = try_html
-        self.factory_class = factory_class or {"http": Request, "websocket": WebSocket}
+        self.allow_underline = allow_underline
+        if basepath is not None:
+            self.basepath = basepath
+        logger.debug(f"Index File in module {module_name}, basepath: {basepath}")
+
+    @cached_property
+    def basepath(self) -> str:
+        return os.path.dirname(
+            os.path.dirname(
+                os.path.abspath(importlib.import_module(self.module_name).__file__)
+            )
+        )
 
     def _split_path(self, path: str) -> typing.List[str]:
         """
@@ -217,119 +245,92 @@ class IndexFile:
                 yield module, path
 
     async def http(self, scope: Scope, receive: Receive, send: Send) -> None:
-        request = self.factory_class["http"](scope, receive, send)
+        request = scope["app"].factory_class["http"](scope, receive, send)
         pathlist = self._split_path(request.url.path)
 
         module = self.get_view(request.url.path)
         if not hasattr(module, "HTTP"):
-            if self.try_html:
-                pathlist = pathlist[1:]  # delete self.module_name from pathlist
-                try:
-                    # only html, no middleware/background tasks or other anything
-                    return await TemplateResponse(
-                        os.path.join(*pathlist) + ".html", {"request": request}
-                    )(scope, receive, send)
-                except LookupError:
-                    pass
             raise NoMatchFound()
 
         get_response = getattr(module, "HTTP")
 
-        try:
-            # set background tasks contextvar
-            after_response_tasks_token = after_response_tasks_var.set(BackgroundTasks())
-            finished_response_tasks_token = finished_response_tasks_var.set(
-                BackgroundTasks()
-            )
-            # call middleware
-            for deep in range(len(pathlist), 0, -1):
-                module = importlib.import_module(".".join(pathlist[:deep]))
-                if not hasattr(module, "Middleware"):
-                    continue
-                get_response = getattr(module, "Middleware")(get_response)
+        # call middleware
+        for deep in range(len(pathlist), 0, -1):
+            module = importlib.import_module(".".join(pathlist[:deep]))
+            if not hasattr(module, "Middleware"):
+                continue
+            get_response = getattr(module, "Middleware")(get_response)
 
-            # get response
-            response = await get_response(request)
-            background_tasks = after_response_tasks_var.get()
-            if background_tasks.tasks != []:
-                response.background = background_tasks
-            await response(scope, receive, send)
-        finally:
-            after_response_tasks_var.reset(after_response_tasks_token)
-
-            run_finished_response_tasks = finished_response_tasks_var.get()
-            finished_response_tasks_var.reset(finished_response_tasks_token)
-            if run_finished_response_tasks.tasks != []:
-                await run_finished_response_tasks()
+        # get response
+        response = await get_response(request)
+        await response(scope, receive, send)
 
     async def websocket(self, scope: Scope, receive: Receive, send: Send) -> None:
-        websocket = self.factory_class["websocket"](scope, receive=receive, send=send)
+        websocket = scope["app"].factory_class["websocket"](
+            scope, receive=receive, send=send
+        )
 
         module = self.get_view(websocket.url.path)
         if not hasattr(module, "Socket"):
             raise NoMatchFound()
 
+        # TODO add middleware
+
         await getattr(module, "Socket")(websocket)
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         handler = getattr(self, scope["type"])
+        url = URL(scope=scope)
 
-        if scope["type"] in ("http", "websocket"):  # Handle some special routes
-            url = URL(scope=scope)
-            path = url.path
-            if path.endswith("/index"):
-                handler = RedirectResponse(
-                    url.replace(path=f'{path[:-len("/index")]}'), status_code=301,
-                )
-            elif "_" in path and not Config().ALLOW_UNDERLINE:
-                handler = RedirectResponse(
-                    url.replace(path=f'{path.replace("_", "-")}'), status_code=301,
-                )
-            elif path == "/favicon.ico":
-                if os.path.exists(os.path.normpath("favicon.ico")):
-                    handler = FileResponse("favicon.ico")
+        if url.path.endswith("/index"):
+            handler = RedirectResponse(
+                url.replace(path=url.path[: -len("/index")]), status_code=301,
+            )
+        elif "_" in url.path and not self.allow_underline:
+            handler = RedirectResponse(
+                url.replace(path=url.path.replace("_", "-")), status_code=301,
+            )
 
         await handler(scope, receive, send)
 
 
 class Index:
-    def __init__(self) -> None:
-        self.config = config = Config()
-        self.factory_class = {"http": Request, "websocket": WebSocket}
-        self.indexfile = IndexFile(
-            "views", here, try_html=True, factory_class=self.factory_class
-        )
+    def __init__(
+        self,
+        *,
+        templates: typing.Iterable[str] = (),
+        try_html: bool = True,
+        mount_apps: typing.List[typing.Tuple[str, ASGIApp]] = [],
+        on_startup: typing.List[typing.Callable] = [],
+        on_shutdown: typing.List[typing.Callable] = [],
+        routes: typing.List[BaseRoute] = [],
+        factory_class: FactoryClass = {"http": Request, "websocket": WebSocket},
+    ) -> None:
+        self.factory_class = factory_class
+        self.router = Router(routes)
+
+        templates_loaders: typing.List[
+            typing.Union[FileSystemLoader, PackageLoader]
+        ] = []
+        for template_path in templates:
+            if ":" in template_path:  # package: "package:path"
+                package_name, package_path = template_path.split(":", maxsplit=1)
+                templates_loaders.append(PackageLoader(package_name, package_path))
+            else:  # normal: "path"
+                templates_loaders.append(FileSystemLoader(template_path))
+
         self.jinja_env = Environment(
-            loader=FileSystemLoader(config.TEMPLATES), enable_async=True
+            loader=ChoiceLoader(templates_loaders), enable_async=True,
         )
+        self.try_html = try_html
+
+        # Shallow copy list to prevent memory leak.
+        self.mount_apps = list(mount_apps)
         self.lifespan = Lifespan(
-            on_startup={
-                check_on_startup.__qualname__: check_on_startup,
-                create_directories.__qualname__: create_directories,
-            },
-            on_shutdown={
-                clear_check_on_shutdown.__qualname__: clear_check_on_shutdown,
-                clear_directories.__qualname__: clear_directories,
-            },
+            on_startup=list(on_startup), on_shutdown=list(on_shutdown)
         )
-        self.mount_apps: typing.List[typing.Tuple[str, ASGIApp]] = [
-            (
-                "/static",
-                StaticFiles(directory=os.path.join(here, "static"), check_dir=False),
-            ),
-        ]
-        self.user_middlewares: typing.List[Middleware] = [
-            Middleware(
-                CORSMiddleware,
-                allow_origins=config.CORS_ALLOW_ORIGINS,
-                allow_methods=config.CORS_ALLOW_METHODS,
-                allow_headers=config.CORS_ALLOW_HEADERS,
-                allow_credentials=config.CORS_ALLOW_CREDENTIALS,
-                allow_origin_regex=config.CORS_ALLOW_ORIGIN_REGEX,
-                expose_headers=config.CORS_EXPOSE_HEADERS,
-                max_age=config.CORS_MAX_AGE,
-            ),
-        ]
+
+        self.user_middlewares: typing.List[Middleware] = []
         self.exception_handlers: typing.Dict[
             typing.Union[int, typing.Type[Exception]], typing.Callable
         ] = {}
@@ -340,7 +341,7 @@ class Index:
         self.asgiapp = self.build_app()
 
     def build_app(self) -> ASGIApp:
-        config = self.config
+        config = Config()
         error_handler = None
         exception_handlers = {}
 
@@ -350,12 +351,35 @@ class Index:
             else:
                 exception_handlers[key] = value
 
-        middlewares = [
-            Middleware(TrustedHostMiddleware, allowed_hosts=config.ALLOWED_HOSTS),
-            Middleware(
-                ServerErrorMiddleware, handler=error_handler, debug=config.DEBUG
-            ),
-        ] + self.user_middlewares
+        middlewares = []
+
+        if config.ALLOWED_HOSTS != ["*"]:
+            middlewares.append(
+                Middleware(TrustedHostMiddleware, allowed_hosts=config.ALLOWED_HOSTS)
+            )
+
+        middlewares.append(
+            Middleware(ServerErrorMiddleware, handler=error_handler, debug=config.DEBUG)
+        )
+
+        if (
+            config.CORS_ALLOW_ORIGIN_REGEX is not None
+            or len(config.CORS_ALLOW_ORIGINS) > 0
+        ):
+            middlewares.append(
+                Middleware(
+                    CORSMiddleware,
+                    allow_origins=config.CORS_ALLOW_ORIGINS,
+                    allow_methods=config.CORS_ALLOW_METHODS,
+                    allow_headers=config.CORS_ALLOW_HEADERS,
+                    allow_credentials=config.CORS_ALLOW_CREDENTIALS,
+                    allow_origin_regex=config.CORS_ALLOW_ORIGIN_REGEX,
+                    expose_headers=config.CORS_EXPOSE_HEADERS,
+                    max_age=config.CORS_MAX_AGE,
+                )
+            )
+
+        middlewares += self.user_middlewares
 
         middlewares.append(Middleware(ExceptionMiddleware, handlers=exception_handlers))
 
@@ -419,71 +443,91 @@ class Index:
         For lifespan, call Index directly.
         For http/websocket, find the appropriate subapp, or Index itself.
         """
-        if scope["type"] in ("http", "websocket"):
-            path = scope["path"]
-            root_path = scope.get("root_path", "")
-            has_received = False
+        if scope["type"] == "lifespan":
+            return await self.lifespan(scope, receive, send)
 
-            async def subreceive() -> Message:
-                nonlocal has_received
-                has_received = True
-                return await receive()
+        path = scope["path"]
+        root_path = scope.get("root_path", "")
+        has_received = False
 
-            async def subsend(message: Message) -> None:
-                if (
-                    message["type"] == "http.response.start"
-                    and message["status"] == 404
-                ):
-                    raise NoMatchFound()
-                await send(message)
+        async def subreceive() -> Message:
+            nonlocal has_received
+            has_received = True
+            return await receive()
 
-            # Call into a submounted app, if one exists.
-            for path_prefix, app in self.mount_apps:
-                if not path.startswith(path_prefix + "/"):
-                    continue
-                if isinstance(app, WSGIMiddleware):
-                    if scope["type"] != "http":
-                        continue
-                subscope = copy.copy(scope)
-                subscope["path"] = path[len(path_prefix) :]
-                subscope["root_path"] = root_path + path_prefix
-                try:
-                    await app(subscope, subreceive, subsend)
-                    return
-                except NoMatchFound:
-                    if has_received:  # has call received
-                        raise HTTPException(404)
-                except HTTPException as e:
-                    if e.status_code != 404 or has_received:
-                        raise e
+        async def subsend(message: Message) -> None:
+            if message["type"] == "http.response.start" and message["status"] == 404:
+                raise NoMatchFound()
+            await send(message)
 
+        # Call into a submounted app, if one exists.
+        for path_prefix, app in filter(
+            lambda item: path.startswith(item[0] + "/"), self.mount_apps
+        ):
+            if isinstance(app, WSGIMiddleware) and scope["type"] != "http":
+                continue
+            subscope = copy.copy(scope)
+            subscope["path"] = path[len(path_prefix) :]
+            subscope["root_path"] = root_path + path_prefix
             try:
-                await self.indexfile(scope, receive, send)
+                await app(subscope, subreceive, subsend)
+                return
             except NoMatchFound:
-                if scope["type"] == "http":
+                if has_received:  # has call received
                     raise HTTPException(404)
-                await WebSocketClose(WS_1001_GOING_AWAY)(scope, receive, send)
+            except HTTPException as e:
+                if e.status_code != 404 or has_received:
+                    raise e
 
-        elif scope["type"] == "lifespan":
-            await self.lifespan(scope, receive, send)
+        handler: typing.Optional[ASGIApp] = None
+
+        try:
+            path_params, endpoint = self.router.search(scope["type"], scope["path"])
+            scope["path_params"] = path_params
+            request = self.factory_class[scope["type"]](scope, receive, send)  # type: ignore
+            get_handler = await parse_params(endpoint, request)
+
+            for middleware in getattr(endpoint, "middlewares", ()):
+                get_handler = middleware(get_handler)
+
+            if scope["type"] == "http":
+                handler = convert(await get_handler(request))
+            else:
+                handler = get_handler(request)
+        except NoMatchFound:
+            if scope["type"] == "http" or self.try_html:
+                # only html, no middleware/background tasks or other anything
+                handler = try_html(self.factory_class["http"](scope, receive, send))
+
+        if handler is None:
+            if scope["type"] == "http":
+                raise HTTPException(404)
+            handler = WebSocketClose(WS_1001_GOING_AWAY)
+
+        return await handler(scope, receive, send)
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         scope["app"] = self
 
-        # Force jump to https/wss
-        if (
-            self.config.FORCE_SSL
-            and scope["type"] in ("http", "websocket")
-            and scope["scheme"] in ("http", "ws",)
-        ):
+        if scope["type"] in ("http", "websocket"):  # Handle some special routes
             url = URL(scope=scope)
-            redirect_scheme = {"http": "https", "ws": "wss"}[url.scheme]
-            netloc = url.hostname if url.port in (80, 443) else url.netloc
-            url = url.replace(scheme=redirect_scheme, netloc=netloc)
-            response = RedirectResponse(
-                url, status_code=301
-            )  # for SEO, status code must be 301
-            await response(scope, receive, send)
-            return
+            response: typing.Optional[Response] = None
+
+            # Force jump to https/wss
+            if Config().FORCE_SSL and scope["scheme"] in ("http", "ws"):
+                redirect_scheme = {"http": "https", "ws": "wss"}[url.scheme]
+                netloc = url.hostname if url.port in (80, 443) else url.netloc
+                url = url.replace(scheme=redirect_scheme, netloc=netloc)
+                response = RedirectResponse(
+                    url, status_code=301
+                )  # for SEO, status code must be 301
+
+            if url.path == "/favicon.ico":
+                if os.path.exists(os.path.normpath("favicon.ico")):
+                    response = FileResponse("favicon.ico")
+
+            if response is not None:
+                await response(scope, receive, send)
+                return
 
         await self.asgiapp(scope, receive, send)
