@@ -1,5 +1,4 @@
 import os
-import sys
 import time
 import signal
 import logging
@@ -12,14 +11,6 @@ import uvicorn
 
 from .utils import import_module
 from .config import here, LOG_LEVELS, Config
-from .applications import Index
-
-try:
-    from .test import cmd_test
-except ImportError as e:
-
-    def cmd_test(raise_exception=e):  # type: ignore
-        raise raise_exception
 
 
 from .__version__ import __version__
@@ -51,18 +42,10 @@ def execute(command: Union[List[str], str]) -> int:
 
 
 @click.group(help=f"Index.py {__version__}")
-@click.option("--env", default=lambda: config.ENV, help="set config.ENV")
-@click.option(
-    "--debug/--no-debug", default=lambda: config.DEBUG, help="set config.DEBUG"
-)
-def main(env, debug):
-    # change config
-    os.environ["INDEX_ENV"] = env
-    os.environ["INDEX_DEBUG"] = "on" if debug else "off"
-    config.import_from_environ()
-    Index().rebuild_app()
+def main():
     # set index logger level
-    logging.getLogger("indexpy").setLevel(LOG_LEVELS[config.LOG_LEVEL])
+    indexpy_logger = logging.getLogger("indexpy")
+    indexpy_logger.setLevel(LOG_LEVELS[config.LOG_LEVEL])
 
 
 @main.command(help="use only uvicorn to deploy")
@@ -114,6 +97,4 @@ def gunicorn(workers, daemon, configuration, method, application):
         execute(["kill -HUP", "`cat .pid`"])
 
 
-main.command(name="test", help="run test")(cmd_test)
-
 import_module("commands")
